--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Antti Lukats, Oleksandr Kiyenko, John Hartfiel
--
-- TEBF808 Master CPLD U17, SC.Sch, first in the JTAG Chain
--
--
-- Revision     : 05
-- Release Date : 2017-06-09
-- Supported PCB Revision: 03,04
-- BUG Comments:
--  SC19 loc constrain is SC17 Pin
-- changes REV04 to REV05:
--  SD WP forwarded second CPLD
--  PL access to XMOD LED
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.STD_LOGIC_arith.all;
use ieee.STD_LOGIC_unsigned.all;
library machxo2;
use machxo2.all;
--------------------------------------------------------------------------------
entity top is
port (
	--M_TMS		 		: in  STD_LOGIC;
	--M_TCK		 		: in  STD_LOGIC;
	--M_TDI		 		: in  STD_LOGIC;
	--M_TDO		 		: out STD_LOGIC;

	--C_TMS		 		: out STD_LOGIC;
	--C_TCK		 		: out STD_LOGIC;
	--C_TDI		 		: out STD_LOGIC;
	--C_TDO		 		: in  STD_LOGIC;
		
	OSC_EN				: out STD_LOGIC;
	I2C_RST				: out STD_LOGIC;
	USB_RST				: out STD_LOGIC;
	--USB_TRST	   		: out STD_LOGIC; -- do not care

	--EN_FMC		 		: out STD_LOGIC;
	FMC_VID0	   		: out STD_LOGIC;
	FMC_VID1	   		: out STD_LOGIC;
	FMC_VID2	   		: out STD_LOGIC;
	FMC_CLKDIR	 		: in  STD_LOGIC;
	
	FMC_TCK	   			: out STD_LOGIC;
	FMC_TMS	   			: out STD_LOGIC;
	FMC_TDI	   			: out STD_LOGIC;
	FMC_TDO	   			: in  STD_LOGIC;
		
	SDA		 			: in  STD_LOGIC;
	SCL		 			: in  STD_LOGIC;				
	SD_WP		 		: in  STD_LOGIC;				
				
	-- 20 pin JTAG
	JTAG_TRST			: in  STD_LOGIC;				
	JTAG_SRST			: in  STD_LOGIC;				 -- soft reset
	JTAG_TCK			: in  STD_LOGIC;				
	JTAG_RTCK			: out STD_LOGIC;				
	JTAG_TMS			: in  STD_LOGIC;				
	JTAG_TDI			: in  STD_LOGIC;				
	JTAG_TDO			: out STD_LOGIC;				
		
	CAN_TX		   		: out STD_LOGIC;
	CAN_RX		   		: in  STD_LOGIC;
	CAN_S		   		: out STD_LOGIC;
	CAN_FAULT   		: in  STD_LOGIC;

	MIO26		   		: in  STD_LOGIC;
	MIO27		   		: in  STD_LOGIC;
	MIO28		   		: in  STD_LOGIC;
	MIO29		   		: in  STD_LOGIC;
		
	SFP_LED0	   		: out STD_LOGIC;
	SFP_LED1	   		: out STD_LOGIC;
	SFP_LED2	   		: out STD_LOGIC;
	SFP_LED3	   		: out STD_LOGIC;
	--
	-- ETH PHY
	--	
	PHY_LED0	   		: in  STD_LOGIC;
	PHY_LED1	   		: in  STD_LOGIC;
	PHY_LED2	   		: in  STD_LOGIC;
	PHY_CLK	   			: in  STD_LOGIC;
	ETH_RST				: out STD_LOGIC;
	PHY_CONFIG	 		: out STD_LOGIC;
	
	JLED1		   		: out STD_LOGIC;
	JLED2A		   		: out STD_LOGIC;
	JLED2B		   		: out STD_LOGIC;
	XMOD_G				: in  STD_LOGIC; --  There is extra Button on XMOD
	XMOD_E				: out STD_LOGIC; --  There is extra LED on XMOD E pin
	
--		XMOD_TXD			: out STD_LOGIC;
--		XMOD_RXD			: in  STD_LOGIC;
	dummy		  		: out STD_LOGIC;

	-- to From DP
	DP_HPD				: in  STD_LOGIC;
	DP_DE				: out STD_LOGIC;
	DP_TX				: out STD_LOGIC;
	DP_RX				: in  STD_LOGIC;
		
-----------------------------------------------------		
	-- USB_TRST			  : out STD_LOGIC;      --(PCB_REV02, REV03,REV04 -> PCB_REV04_DET and input), REV02:uncomment and loc constrain
	-- PCB_REV04_DET		: in STD_LOGIC;      --(REV02->USB_TRST and output, PCB_REV03->float, REV04->GND ), REV02:comment 
-- REV 3 PCB only
	USBH_RST			    : out STD_LOGIC;      --(PCB_REV02, NC--> other USB HUB), REV02:comment 
	USBH_MODE0	   		: out STD_LOGIC;  --(PCB_REV02, NC--> other USB HUB), REV02:comment 
	USBH_MODE1	   		: out STD_LOGIC;  --(PCB_REV02, NC--> other USB HUB), REV02:comment 
-----------------------------------------------------
-- PL Side
--		PL_DP_HPD	  		: out STD_LOGIC;  -- IO 14
--		PL_DP_DE	   		: in  STD_LOGIC;   -- IO 15
--		PL_DP_TX	   		: in  STD_LOGIC;   -- IO 16
--		PL_DP_RX	   		: out STD_LOGIC;  -- IO 17

	DP_EN				: out STD_LOGIC; -- Enable power to DP out LDO
	SC10				: inout STD_LOGIC;  -- DP AUX
	SC11				: in  STD_LOGIC;  -- DP EN
	SC12				: out STD_LOGIC;  -- -- HPD shared?
	SC13				: out STD_LOGIC;  -- TX data
	SC14				: in  STD_LOGIC;  -- RX DATA
	SC15				: in  STD_LOGIC;  -- RX CLK

	SC18				: in  STD_LOGIC;  -- 
	SC19				: in  STD_LOGIC;  -- forwarded to XMOD LED

	CLK0				: in  STD_LOGIC;  -- --(PCB_REV02, differential P), REV02:comment 
	CLK1				: in  STD_LOGIC;  -- --(PCB_REV02, differential N), REV02:comment 

	-- inter SCx
	X0	  				: in  STD_LOGIC;  -- 
	X1	  				: in  STD_LOGIC;  -- 
	X2	  				: in  STD_LOGIC;  -- 
	X3	  				: in  STD_LOGIC;  -- 
	X4	  				: out STD_LOGIC;  -- SD WP to slave cpld
	X5	  				: out STD_LOGIC;  -- 
	X6	  				: out STD_LOGIC;  -- 
	X7	  				: out STD_LOGIC;  -- 
	SW3		 			: in  STD_LOGIC;   -- 
	SW4		 			: in  STD_LOGIC;   -- 
	LED0				: out STD_LOGIC;
	LED1				: out STD_LOGIC;
	-- SFP Controls
	SFP1_TX_DIS			: out STD_LOGIC; --(PCB_REV02, NC), REV02:comment 
	SFP2_TX_DIS			: out STD_LOGIC --(PCB_REV02, NC), REV02:comment 
 );
end entity top;
--------------------------------------------------------------------------------
architecture translated of top is
--------------------------------------------------------------------------------
component ddsrpi_slave is
port ( 
	RX_CLK		: in  STD_LOGIC;
	RX_DATA		: in  STD_LOGIC;
	TX_DATA		: out STD_LOGIC;
	user_clk	: out STD_LOGIC;
	gpio_out	: out STD_LOGIC_VECTOR(31 downto 0);
	gpio_in		: in  STD_LOGIC_VECTOR(31 downto 0);
	resetn		: in  STD_LOGIC;
	ready		: out STD_LOGIC
);
end component ddsrpi_slave;

component OSCH
-- synthesis translate_off
generic (
	NOM_FREQ: string := "24.18"
);
-- synthesis translate_on
port ( 
	STDBY		: in  STD_LOGIC;
	OSC			: out STD_LOGIC;
	SEDSTDBY 	: out STD_LOGIC
);
end component OSCH;

attribute NOM_FREQ 		: string;
attribute NOM_FREQ of OSCinst0 : label is "24.18";

signal osc_clk			: STD_LOGIC;
signal clk				: STD_LOGIC;
signal cnt				: STD_LOGIC_VECTOR(31 downto 0);
signal LED_Blink		: STD_LOGIC;
signal blink			: STD_LOGIC;
signal por_reset		: STD_LOGIC; -- if this is HIGH assert reset to all peripherals
signal por_resetn		: STD_LOGIC; -- if this is LOW assert reset to all peripherals
signal rgpio_clk 		: STD_LOGIC; 		
signal rgpio_txd 		: STD_LOGIC; 		
signal rgpio_rxd 		: STD_LOGIC; 		
signal rgpio_out 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal rgpio_in 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal rgpio_active 	: STD_LOGIC; 		

signal startup_cnt 		: STD_LOGIC_VECTOR(31 downto 0);
signal usb_rst_cnt 		: STD_LOGIC_VECTOR(31 downto 0);
signal usb_hub_rstn		: STD_LOGIC;
signal usb_phy_rstn		: STD_LOGIC;
--------------------------------------------------------------------------------
BEGIN
--------------------------------------------------------------------------------
--REV02 only:
-- USB_TRST	<= '1';     --(PCB_REV02, REV03,REV04 -> PCB_REV04_DET and input), REV02:add 
--------------------------------------------------------------------------------
-- SW4 control the FMC VADJ
-- 010 = 1.8V
-- 101	= 1.2V
FMC_VID0	<= '0' when SW4 ='0' else '1';
FMC_VID1	<= '1' when SW4 ='0' else '0';
FMC_VID2	<= '0' when SW4 ='0' else '1';
---- Sanity check to other CPLD
X6 			<= '0';
X7			<= '1';
-- if 00 or 11 are sampled, other CPLD
-- must turn off FMC_VADJ regulator just in case
-- the VIDx is invalid
--------------------------------------------------------------------------------
process(rgpio_active, usb_hub_rstn, por_resetn, rgpio_out)
begin
	if(rgpio_active = '1')then
		USBH_RST	<= (not rgpio_out(1)) and usb_hub_rstn and XMOD_G; --, REV02:comment 
		USB_RST		<= (not rgpio_out(0)) and usb_phy_rstn and XMOD_G;
	else
		USBH_RST	<= usb_hub_rstn and XMOD_G; --, REV02:comment 
		USB_RST		<= usb_phy_rstn and XMOD_G;
	end if;
end process;
--------------------------------------------------------------------------------
I2C_RST	 	  <= (not rgpio_out(2)) when rgpio_active = '1' else por_resetn;
ETH_RST  	  <= (not rgpio_out(3)) when rgpio_active = '1' else por_resetn;
USBH_MODE0	<= (not rgpio_out(4)) when rgpio_active = '1' else '1'; --, REV02:comment 
USBH_MODE1	<= (not rgpio_out(5)) when rgpio_active = '1' else '1'; --, REV02:comment 
--------------------------------------------------------------------------------
LED0 		    <= rgpio_out(6) when rgpio_active = '1' else usb_hub_rstn;	--X0;  -- GREEN
LED1		    <= rgpio_out(7) when rgpio_active = '1' else por_reset;
SFP_LED0	  <= rgpio_out(8) when rgpio_active = '1' else blink when (X0='1' and X1='1') else '0';
SFP_LED1	  <= rgpio_out(9) when rgpio_active = '1' else blink when (X0='1' and X1='1') else '0';
SFP_LED2	  <= rgpio_out(10) when rgpio_active = '1' else blink when (X0='1' and X1='1') else '0';
SFP_LED3	  <= rgpio_out(11) when rgpio_active = '1' else blink when (X0='1' and X1='1') else '0';
JLED1		    <= rgpio_out(12) when rgpio_active = '1' else blink when (X0='1' and X1='1') else (not PHY_LED0) when X1='0' else '0';
JLED2A		  <= rgpio_out(13) when rgpio_active = '1' else blink when (X0='1' and X1='1') else PHY_LED1 when X1='0' else '0';
JLED2B		  <= (not rgpio_out(14)) when rgpio_active = '1' else (not blink) when (X0='1' and X1='1') else '0';
---------------------------------------
X4 <= SD_WP;
---------------------------------------
PHY_CONFIG 	<= '1';
---------------------------------------
XMOD_E 		  <= blink when por_reset = '1' else SC19;
SC10		    <= DP_RX when SC11 = '0' else 'Z';
DP_TX		    <= SC10; 
DP_DE 		  <= SC11; -- 1 = transmit
SC12		    <= DP_HPD;
---------------------------------------
JTAG_RTCK	  <= JTAG_TCK;
JTAG_TDO	  <= JTAG_TDI;
---------------------------------------
OSC_EN		  <= '1'; -- for the case we use it :)
---------------------------------------
CAN_TX		  <= '0';
CAN_S		    <= '0';

---------------------------------------
por_reset	  <= '1' when (X0='0' or X1='1') else '0';
por_resetn	<= not por_reset;
---------------------------------------
--endless counter
process(clk)
begin
	if (rising_edge(clk)) then
		cnt <= cnt + X"00000001";		
	end if;
end process;
blink 		  <= cnt(23);

--------------------------------------------------------------------------------
process(clk)
begin
	if(por_reset = '1')then
		startup_cnt				<= (others => '0');
		usb_hub_rstn			<= '0';
		usb_phy_rstn			<= '0';
	elsif(clk = '1' and clk'event)then
		if(startup_cnt /= x"FFFF_FFFF")then		-- End of sequence
			startup_cnt			<= startup_cnt + 1;
		end if;
		if(startup_cnt >= x"0100_0000")then
			usb_phy_rstn			<= '1';		
		else
			usb_phy_rstn			<= '0';
		end if;
		if(startup_cnt >= x"2500_0000")then
			usb_hub_rstn			<= '1';		
		else
			usb_hub_rstn			<= '0';
		end if;
	end if;
end process;
--------------------------------------------------------------------------------
--internal CLK
OSCInst0: OSCH
-- synthesis translate_off
GENERIC MAP ( 
	NOM_FREQ => "24.18" 
)
-- synthesis translate_on
PORT MAP (
	STDBY		=> '0', 
	OSC			=> osc_clk, 
	SEDSTDBY	=> open
);
clk 		<= osc_clk;
--------------------------------------------------------------------------------

--
-- RGPIO Pin Mapping
--
rgpio_in(0)		<= SW3; --
rgpio_in(1) 	<= SW4; --
rgpio_in(2) 	<= SD_WP; --
rgpio_in(3) 	<= XMOD_G; --
rgpio_in(4) 	<= MIO26; --
rgpio_in(5) 	<= MIO27; --
rgpio_in(6) 	<= MIO28; --
rgpio_in(7) 	<= MIO29; --
rgpio_in(8) 	<= CAN_FAULT; --
rgpio_in(9) 	<= PHY_LED0; --
rgpio_in(10)	<= PHY_LED1; --
rgpio_in(11)	<= PHY_LED2; --
rgpio_in(12)	<= FMC_TDO; --
rgpio_in(13)	<= FMC_CLKDIR; -- we do not need it really
rgpio_in(14)	<= JTAG_TRST; -- we do not need it really
rgpio_in(15)	<= JTAG_SRST; --
rgpio_in(16)	<= JTAG_TCK; --
rgpio_in(17)	<= JTAG_TDI; --
rgpio_in(18)	<= JTAG_TMS; --
rgpio_in(19)	<= DP_HPD; --
rgpio_in(20)	<= SDA; --
rgpio_in(21)	<= SCL; --
rgpio_in(22)	<= '0';
rgpio_in(23)	<= '0';
rgpio_in(27 downto 24)	<= "0000"; -- reserved
rgpio_in(31 downto 28)	<= "1010"; -- interface detection


--rgpio_in(20) <= SDA; --
--rgpio_in(21) <= SDA; --
rgpio_clk 		<= SC15;
SC13 			<= rgpio_txd;
rgpio_rxd 		<= SC14;
--------------------------------------------------------------------------------
MyRemoteGPIO: ddsrpi_slave
PORT MAP ( 
	RX_CLK		=> rgpio_clk,
	TX_DATA 	=> rgpio_txd,
	RX_DATA 	=> rgpio_rxd,
	user_clk	=> open,  
	resetn 		=> '1',	
	ready 		=> open,
	--valid		=> open,
	gpio_in		=> rgpio_in,
	gpio_out	=> rgpio_out
);
rgpio_active 	<= '1' when rgpio_out(31 downto 28) = "1010" else '0';
--------------------------------------------------------------------------------
SFP1_TX_DIS		<= '0'; --, REV02:comment 
SFP2_TX_DIS		<= '0'; --, REV02:comment 
--------------------------------------------------------------------------------
dummy 	<= PHY_CLK or CLK0 or CLK1 or CAN_RX or SC18 or X2 or X3;
--------------------------------------------------------------------------------
END ARCHITECTURE translated;