--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Antti Lukats, Oleksandr Kiyenko, John Hartfiel
--
-- TEBF808 Slave CPLD U39, SC.Sch, second in the JTAG Chain
--
--
-- Revision     : 06
-- Release Date : 2017-06-09
-- Supported PCB Revision: 02(Pin-126 SC_IO8 not connected),03,04
-- changes REV04 to REV05:
--  PS can be rebooted by FSBL over MIO30, for proper pcie initialisation without Reset Button
--  SD Boot from micoSD only if switch is selected  
--  RGPIO connection 
--  SD WP to FPGA
--  debounce power/rst button
--  MIO and PL LED access
--  Needed changes since Lattice Diamond 3.9 (change date 2017.07.10) (CPLD Revision is not changed for this source code bugfix)
--  - rename label for Reset/Power rebounded "db_proc" "db_cnt_proc" to "db_1/2_proc" "db_1/2_cnt_proc" (this was ignored on older versions) 
--  - rename CLK constrains osc_clk to osc_clk_0 on constrain file.
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.STD_LOGIC_arith.all;
use ieee.STD_LOGIC_unsigned.all;
library machxo2;
use machxo2.all;
--------------------------------------------------------------------------------
ENTITY top IS
PORT (
	--M_TMS		 		: in  STD_LOGIC;
	--M_TCK		 		: in  STD_LOGIC;
	--M_TDI		 		: in  STD_LOGIC;
	--M_TDO		 		: out STD_LOGIC;

	--C_TMS		 		: out STD_LOGIC;
	--C_TCK		 		: out STD_LOGIC;
	--C_TDI		 		: out STD_LOGIC;
	--C_TDO		 		: in  STD_LOGIC;
		
  MIO44       : out STD_LOGIC; --SD WP
    
	MIO42				: out STD_LOGIC;
	MIO43				: in  STD_LOGIC;
  MIO40       : in std_logic; --to PWRLED_P / LED_P
		
		
	XMOD_TXD			: out STD_LOGIC;
	XMOD_RXD			: in  STD_LOGIC;
	XMOD_BTN			: in  STD_LOGIC;
	XMOD_LED			: out  STD_LOGIC;

	SCL					: in  STD_LOGIC;
	SDA					: in  STD_LOGIC;
	PWR_BTN				: in  STD_LOGIC;
	RST_BTN				: in  STD_LOGIC;

	PERSTn				: out STD_LOGIC; --PCIe Reset
	MRESETn				: out STD_LOGIC;
	PROG_B				: INOUT STD_LOGIC;

	-- 1.8V
	MODE0				: out STD_LOGIC;
	MODE1				: out STD_LOGIC;
	MODE2				: out STD_LOGIC;
	MODE3				: out STD_LOGIC;
	ERROR				: in  STD_LOGIC;
	ERR_STAT			: in  STD_LOGIC;
	SRSTn				: out STD_LOGIC;
	DONE				: in  STD_LOGIC;
	INIT				: in  STD_LOGIC;
	PLL_RSTn			: out STD_LOGIC;
	PLL_SEL0			: out STD_LOGIC;
	PLL_SEL1			: out STD_LOGIC;
	PLL_LOL				: in  STD_LOGIC;
	MIO30				: in  STD_LOGIC; --force reboot after FSBL-PLL configuration 
	MIO31				: in  STD_LOGIC; --pcie reset
	----------------------
	PG_PL				: in  STD_LOGIC;
	PG_DDR				: in  STD_LOGIC;
	PG_FPD				: in  STD_LOGIC;
	PG_LPD				: in  STD_LOGIC;
	PG_GT_L				: in  STD_LOGIC;
	PG_GT_R				: in  STD_LOGIC;
	PG_PSGT				: in  STD_LOGIC;
	PG_PLL				: in  STD_LOGIC;

	PCI_SFP_EN			: out STD_LOGIC;
	MOD_EN				: out STD_LOGIC;
	PER_EN				: out STD_LOGIC;
	EN_1V8				: out STD_LOGIC;
	EN_LPD				: out STD_LOGIC;
	EN_FPD				: out STD_LOGIC;
	EN_PL				: out STD_LOGIC;
	EN_DDR				: out STD_LOGIC;
	EN_PLL_PWR			: out STD_LOGIC;

	EN_PSGTR			: out STD_LOGIC;
	EN_GT_R				: out STD_LOGIC;
	EN_GT_L				: out STD_LOGIC;


	LED_N				: out STD_LOGIC; 
	LED_P				: out STD_LOGIC; 
		
	HDLED_N				: out STD_LOGIC; 
	HDLED_P				: out STD_LOGIC; 

	-- REV 02
	SD_CD_S				: in  STD_LOGIC;
	SD_CD_B				: in  STD_LOGIC;
	SD_CD_OUT			: out STD_LOGIC;
	SD_SEL				: out STD_LOGIC;  -- Select microSD or normal SD
	SD_A_EN				: out STD_LOGIC;
	SD_B_EN				: out STD_LOGIC;
		
	FMC_EN				: out STD_LOGIC; -- Enable FMC Power DANGER!!
	FMC_FAN_EN   	: out STD_LOGIC;
  POK_FMC       : in std_logic;
  -- FMC_PG_C2M    : out std_logic;

	SC0					: in  STD_LOGIC;  -- 
	SC1					: in  STD_LOGIC;  -- 
	SC2					: in  STD_LOGIC;  -- 
	SC3					: out STD_LOGIC;  -- 
	SC4					: out STD_LOGIC;  -- 
	SC5					: out STD_LOGIC;  -- RGPIO
	SC6					: in  STD_LOGIC;  -- RGPIO
	SC7					: in  STD_LOGIC;  -- RGPIO
	-- inter SCx
	X0	 	 			: out STD_LOGIC;  -- 
	X1	  				: out STD_LOGIC;  -- 
	X2	  				: out STD_LOGIC;  -- 
	X3	  				: out STD_LOGIC;  -- 
	X4	  				: in  STD_LOGIC;  -- MMC SD WP from master
	X5	  				: in  STD_LOGIC ;  --
	X6	  				: in  STD_LOGIC;  -- 
	X7	  				: in  STD_LOGIC;  -- 
	F1PWM		  		: out STD_LOGIC;
	F1SENSE	 	 		: in  STD_LOGIC;
	PSON				: out STD_LOGIC; --negative
	PWROK				: in  STD_LOGIC;
	AUD_CLK				: out STD_LOGIC; -- 
	MEMS_CLKIN			: in  STD_LOGIC;  -- 
	dummy				: out STD_LOGIC; -- 
	----------------------------------------------------
	SW1					: in  STD_LOGIC;
	SW2					: in  STD_LOGIC;
	LED2				: out STD_LOGIC;
	LED3				: OUT STD_LOGIC
);
END ENTITY top;
--------------------------------------------------------------------------------
ARCHITECTURE translated OF top IS

--------------------------------------------------------------------------------
COMPONENT PORESET
PORT(
	SYSRST		: in  STD_LOGIC;
	CLK 		: in  STD_LOGIC;		  
	RST 		: OUT STD_LOGIC
);
END COMPONENT;

component ddsrpi_slave is
port ( 
	RX_CLK		: in  STD_LOGIC;
	RX_DATA		: in  STD_LOGIC;
	TX_DATA 	: out STD_LOGIC;
	user_clk 	: out STD_LOGIC;
	gpio_out 	: out STD_LOGIC_VECTOR (31 downto 0);
	gpio_in 	: in  STD_LOGIC_VECTOR (31 downto 0);
	resetn 		: in  STD_LOGIC;
	ready 		: out STD_LOGIC
);
end component ddsrpi_slave;

COMPONENT OSCH
-- synthesis translate_off
GENERIC (
	NOM_FREQ	: string := "24.18"
);
-- synthesis translate_on
PORT ( 
	STDBY 		: in  STD_LOGIC;
	OSC 		: out STD_LOGIC;
	SEDSTDBY	: out STD_LOGIC
);
END COMPONENT OSCH;
component por_b_rst is 
port ( 
	nreset		: in  STD_LOGIC;
	clk 		: in  STD_LOGIC;
	ce  		: in  STD_LOGIC;
	por_b 		: out STD_LOGIC
);
end component;

 attribute NOM_FREQ : string;
 attribute NOM_FREQ of OSCinst0 : label is "24.18";

constant DB_MAX : integer := 22;

signal osc_clk		: STD_LOGIC;
signal clk			: STD_LOGIC;
signal cnt			: STD_LOGIC_VECTOR(31 downto 0);
signal cnt_pwr		: STD_LOGIC_VECTOR(15 downto 0);
signal LED_Blink	: STD_LOGIC;
--signal dummy: STD_LOGIC;
--signal LED_short: STD_LOGIC;
--signal LED_long: STD_LOGIC;
--signal LED_fast: STD_LOGIC;
--signal POWER_OK: STD_LOGIC;
--signal POR_i: STD_LOGIC;
signal POR_B_i		: STD_LOGIC;
--signal RESIN_LATCHED: STD_LOGIC;
--signal srst_sr: STD_LOGIC_VECTOR(5 downto 0);
signal nreset_i		: STD_LOGIC;
signal tick			: STD_LOGIC;
signal blink		: STD_LOGIC;
signal slow_blink	: STD_LOGIC;
signal DET_POWROK	: STD_LOGIC; -- if other CPLD does not seem to be programmed.. force FMC POWER OFF
signal rgpio_clk 	: STD_LOGIC; 		
signal rgpio_txd 	: STD_LOGIC; 		
signal rgpio_rxd 	: STD_LOGIC; 		
signal rgpio_out	: STD_LOGIC_VECTOR(31 downto 0); 		
signal rgpio_in 	: STD_LOGIC_VECTOR(31 downto 0); 
signal rgpio_active : STD_LOGIC; 		
signal bmode_i 		    : STD_LOGIC_VECTOR(3 downto 0); 		
signal SW 			    : STD_LOGIC_VECTOR(1 downto 0); 		
signal SD_detect_n	: STD_LOGIC; 		
signal POR_DONE		: STD_LOGIC;
signal PSON_i 		: STD_LOGIC;
signal mode_blink	: STD_LOGIC;
signal led_cnt		: STD_LOGIC_VECTOR(31 downto 0);
signal perstn_i	  : STD_LOGIC;
signal	pwr_btn_i				:std_logic;
signal	rst_btn_i				:std_logic;

signal	po_rst_i				:std_logic_vector(5 downto 0);
signal	c_force_reboot_i				:std_logic;
signal	n_force_reboot_i				:std_logic;

signal c_pwr_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal n_pwr_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal button_pwr_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_pwr_rst : std_logic;

signal c_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal n_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal button_rst_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_rst_rst : std_logic;

--------------------------------------------------------------------------------
BEGIN
--------------------------------------------------------------------------------
dummy 		<= F1SENSE or INIT or DONE or SCL or SDA or ERROR or ERR_STAT or PROG_B or PG_PL or PG_DDR or PG_FPD or PG_LPD or PG_GT_L or PG_GT_R or PG_PSGT or PG_PLL or SC0 or SC1 or SC2 or X5;
--------------------------------------------------------------------------------
--debounce power button
button_pwr_db(0) <= PWR_BTN;
db_1_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_pwr_db(2 downto 1)		<= button_pwr_db(1 downto 0);
    if(c_pwr_db_cnt(DB_MAX)='1') then
      button_pwr_db(3) <= button_pwr_db(2);
    end if;
	end if;
end process;
db_pwr_rst <= button_pwr_db(2) xor button_pwr_db(1);

db_1_cnt_proc: process(clk)
begin
  if(db_pwr_rst='1') then
    c_pwr_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_pwr_db_cnt 	<= n_pwr_db_cnt;
    end if;
  end if;
end process;
n_pwr_db_cnt <= c_pwr_db_cnt + '1' when c_pwr_db_cnt(DB_MAX)='0' else c_pwr_db_cnt;
------------------------------
--debounce rst button
button_rst_db(0) <= RST_BTN;
db_2_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_rst_db(2 downto 1)		<= button_rst_db(1 downto 0);
    if(c_rst_db_cnt(DB_MAX)='1') then
      button_rst_db(3) <= button_rst_db(2);
    end if;
	end if;
end process;
db_rst_rst <= button_rst_db(2) xor button_rst_db(1);

db_2_cnt_proc: process(clk)
begin
  if(db_rst_rst='1') then
    c_rst_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_rst_db_cnt 	<= n_rst_db_cnt;
    end if;
  end if;
end process;
n_rst_db_cnt <= c_rst_db_cnt + '1' when c_rst_db_cnt(DB_MAX)='0' else c_rst_db_cnt;
--------------------------------------------------------------------------------
pwr_btn_i <= button_pwr_db(3);
rst_btn_i <= button_rst_db(3) and PSON_i and po_rst_i(po_rst_i'length-1);
--------------------------------------------------------------------------------
--reforce reset over modified fsbl, if pcie is detected.
process(cnt,PSON_i)
begin
  if (PSON_i='0') then
    c_force_reboot_i <= '0';
  else
    if (rising_edge(clk)) then
      c_force_reboot_i <= n_force_reboot_i;
    end if;
  end if;
end process;
n_force_reboot_i <= '1' when MIO30='1' else c_force_reboot_i;

process(cnt,c_force_reboot_i)
begin
  if (c_force_reboot_i='0') then
    po_rst_i <= (others => '0');
    po_rst_i(po_rst_i'length-1 downto po_rst_i'length/2) <= (others => '1');
  else
    if (rising_edge(cnt(22))) then
      po_rst_i <= po_rst_i(po_rst_i'length-2 downto 0) & '1';
    end if;
  end if;
end process;

--------------------------------------------------------------------------------
PSON 		<= not PSON_i;
--Sanity check from other CPLD 
DET_POWROK	<= (X6 xor X7) and PSON_i;
FMC_EN		<= DET_POWROK;		
--
PCI_SFP_EN 	<= '1';
MOD_EN 		<= '1';
PER_EN 		<= '1';
EN_1V8 		<= '1';
EN_LPD 		<= '1';
EN_FPD 		<= '1';
EN_PL 		<= '1';
EN_DDR 		<= '1';
EN_PLL_PWR 	<= '1';
EN_PSGTR 	<= '1';
EN_GT_R	 	<= '1';
EN_GT_L	 	<= '1';

PLL_SEL0 	<= '0';
PLL_SEL1 	<= '0';
--------------------------------------------------------------------------------
--internal CLK
OSCInst0: OSCH
-- synthesis translate_off
GENERIC MAP ( 
	NOM_FREQ => "24.18" 
)
-- synthesis translate_on
PORT MAP (
	STDBY		=> '0', 
	OSC			=> osc_clk, 
	SEDSTDBY	=> open
);
clk 	<= osc_clk;

process(clk)
begin
	if (rising_edge(clk)) then
		cnt <= cnt + X"00000001";
		if cnt(17 downto 0) = "100000000000000000" then 
			tick <= '1';
		else
			tick <= '0';
		end if;			
	end if;
end process;

process(clk)
begin
	if rising_edge(clk) then
		led_cnt 	<= led_cnt + X"00000001";
	end if;
end process;

blink 		<= cnt(23);
slow_blink  <= cnt(25);
--------------------------------------------------------------------------------
-- Power/Reset management
Inst_PORESET: PORESET 
PORT MAP( 		
	SYSRST 		=> '1', 		
	RST 		=> POR_DONE, 		
	CLK 		=> Clk 	
);

process(clk)
begin
	if (rising_edge(clk)) then
		if(POR_DONE = '0')then
			PSON_i <= '0';
		else 
			if pwr_btn_i = '0' and PSON_i = '0' then
				if tick='1' then
					cnt_pwr <= cnt_pwr + X"0001";
					if cnt_pwr = "0100" then
						PSON_i <= '1';
					end if;
				end if;
			end if;

			if pwr_btn_i = '0' and PSON_i = '1' then
				if tick='1' then
					cnt_pwr <= cnt_pwr + X"0001";
					if cnt_pwr = "0100" then
						PSON_i <= '0';
					end if;
				end if;
			end if;
			
			if pwr_btn_i = '1' then
				cnt_pwr <= X"0000";
			end if;
		end if;
	
	end if;
end process;
--------------------------------------------------------------------------------
--Boot Mode
-- 0000 JTAG
-- 0010 SPI32
-- 0101 SD1
-- 0110 eMMC
-- 1000 PJTAG0
SW(0) 		<= SW1;
SW(1) 		<= SW2;

bmode_i	   <= 
	"0101" when  (SW = "00" and SD_detect_n='0') else 
	"0010" when  (SW = "00" and SD_detect_n='1')  else  -- SPI when no SD Card
	"0110" when  (SW = "01")  else  -- eMMC
	"1000" when  (SW = "10")  else -- PJTAG0
	"0000"; -- JTAG MODE

MODE0 		<= bmode_i(0);
MODE1 		<= bmode_i(1);
MODE2 		<= bmode_i(2);
MODE3 		<= bmode_i(3);

process(clk)
begin
	if rising_edge(clk) then
		case bmode_i is
			when "0101" => 		-- SD Boot
				mode_blink		<= led_cnt(25) and led_cnt(24) and led_cnt(23) and led_cnt(22);		-- *ooooooo
			when "0010" => 		-- SPI Boot
				mode_blink		<= led_cnt(25) and led_cnt(24) and led_cnt(22);						-- **oooooo
			when "0110" => 		-- eMMC
				mode_blink		<= led_cnt(25) and (led_cnt(24) or led_cnt(23)) and led_cnt(22);	-- ***ooooo
			when "1000" => 		-- PJTAG0
				mode_blink		<= led_cnt(25) and led_cnt(22);										-- ****oooo
			when "0000" => 		-- JTAG
				mode_blink		<= (led_cnt(25) or (led_cnt(24) and led_cnt(23))) and led_cnt(22);	-- *****ooo
			when others	=>		-- Error
				mode_blink		<= led_cnt(22);														-- ********
		end case;
	end if;
end process;
--------------------------------------------------------------------------------
--prog_b not used: attention on RGPIO!
PROG_B 		<=  'Z';
--------------------------------------------------------------------------------
--PS reset
SRSTn 		<= '1';
MRESETn		<= rst_btn_i;
--------------------------------------------------------------------------------
PLL_RSTn 	<= (not rgpio_out(0)) when rgpio_active = '1' else '1';
perstn_i	<= ((not rgpio_out(1)) and MIO31) when rgpio_active = '1' else rst_btn_i and MIO31; 
PERSTn    <= perstn_i;
-----------------------------------------------------------------------
--FMC FAN
FMC_FAN_EN	<= rgpio_out(2) when rgpio_active = '1' else PSON_i;
--------------------------------------------------------------------------------
--LED
LED2 		  <= rgpio_out(12) when rgpio_active = '1' else slow_blink when PSON_i = '0' else '1'; 
LED3 		  <= rgpio_out(13) when rgpio_active = '1' else not rst_btn_i or mode_blink; 	-- ON when RESET or show bootmode
LED_N 		<= rgpio_out(14) when rgpio_active = '1' else '0';
LED_P 		<= (not rgpio_out(15)) when rgpio_active = '1' else slow_blink when PSON_i = '0' else MIO40;
HDLED_N		<= rgpio_out(16) when rgpio_active = '1' else '0';
HDLED_P		<= (not rgpio_out(17)) when rgpio_active = '1' else SC0;
XMOD_LED  <= DONE;
--------------------------------------------------------------------------------
--Audio CLK
AUD_CLK 	<= MEMS_CLKIN;
--------------------------------------------------------------------------------
--reset to Master CPLD
X0 			<= PSON_i;
X1 			<= not rst_btn_i;
--------------------------------------------------------------------------------
-- SD
SD_detect_n <= SD_CD_S and SD_CD_B; --Any SD inserted?
SD_SEL		<= '0' when SD_CD_S = '0' else '1'; -- MicroSD has higher priority!
SD_CD_OUT	<= SD_detect_n; -- DETECTED
SD_A_EN		<= PSON_i;
SD_B_EN		<= PSON_i;

MIO44 <= '1'when SD_CD_S = '0' else X4;
--------------------------------------------------------------------------------
-- UART	
XMOD_TXD	<= MIO43;
MIO42 		<= XMOD_RXD; --
--------------------------------------------------------------------------------
--
-- RGPIO Pin Mapping
--
rgpio_in(0) <= SW1; -- S5-1
rgpio_in(1) <= SW2; -- S5-2
rgpio_in(2) <= XMOD_BTN; -- extra button G
rgpio_in(3) <= c_force_reboot_i; --
rgpio_in(4) <= SD_CD_S; --
rgpio_in(5) <= SD_CD_B; --
rgpio_in(6) <= Error; --    PS ERROR
rgpio_in(7) <= ERR_STAT; -- PS ERROR_STAT
rgpio_in(11 downto 8) <= bmode_i; --current boot mode
rgpio_in(12)  <= PG_LPD; --
rgpio_in(13)  <= PG_DDR; --
rgpio_in(14)	<= PG_FPD; 
rgpio_in(15)	<= PG_PSGT; 
rgpio_in(16)	<= PG_GT_L; 
rgpio_in(17)	<= PG_GT_R; 
rgpio_in(18)	<= POK_FMC; 
rgpio_in(19)	<= DET_POWROK; 
rgpio_in(20)	<= PWROK; 
rgpio_in(21)	<= PG_PL; 
rgpio_in(22)	<= PG_PLL; 
rgpio_in(23)	<= PLL_LOL; 
rgpio_in(27 downto 24)	<= "0000"; -- reserved
rgpio_in(31 downto 28)	<= "1010"; -- interface detection
rgpio_clk 	<= SC6; 
SC5 		<= rgpio_txd;
rgpio_rxd 	<= SC7;

--------------------------------------------------------------------------------
--rgpio
MyRemoteGPIO: ddsrpi_slave
PORT MAP ( 
	RX_CLK		=> rgpio_clk,
	TX_DATA 	=> rgpio_txd,
	RX_DATA 	=> rgpio_rxd,
	user_clk	=> open,  
	resetn 		=> '1',	
	ready 		=> open,
	--valid		=> open,
	gpio_in		=> rgpio_in,
	gpio_out	=> rgpio_out
);
--rgpio_out(27 downto 24) --reserved
rgpio_active <= '1' when rgpio_out(31 downto 28) = "1010" else '0';
--------------------------------------------------------------------------------
END ARCHITECTURE translated;